module Srnggen
  module Comb
    module Aliased
      def to_sys
        "ref(" + @alias[7..-1] + ")"
      end
    end

    module Epsilon
      def to_sys
        "epsilon()"
      end
    end

    module Atom
      def to_sys
        if weight==1 then
          "atom()"
        else
          (["atom()"]*weight).inject {|s, n| "sum(#{s}, #{n})"}
        end
      end
    end

    class Mark
      include Atom
    end

    module Sequence
      def to_sys
        e = elem.to_sys
        "prod(#{e}, seq(#{e}))"
      end
    end

    module Product
      def to_sys
        elems.collect {|e| e.to_sys}.inject {|p, e| "prod(#{p}, #{e})"}
      end
    end
    
    module Union
      def to_sys
        elems.collect {|e| e.to_sys}.inject {|s, e| "sum(#{s}, #{e})"}
      end
    end

    module Reference
      def to_sys
        "ref(" + ref[7..-1] + ")"
      end
    end
  end

  class Document
    def to_sys f
      reste = @defs.keys - @eq
      eq = (@eq+reste).sort{|n,m| n[7..-1].to_i <=> m[7..-1].to_i}
      f.puts <<"EOF"
#include <stdio.h>
#include <stdlib.h>
#include "../src/combstruct.h"

int main(int argc, char **argv)
{
  comb_sys sys;
  double z, *y;

  sys = empty_sys();
#{eq.collect {|n| "  add_sys(sys, #{@defs[n].to_sys}); // #{n}"}.join("\n")}
  add_sys(sys, #{root.to_sys});
  y = malloc(sys_len(sys)*sizeof(double));
  z = sing_sys(sys, 10e-8, 10e-5, y);
  printf(\"start=%.10lf\\n\", y[#{eq.size}]);
  printf(\"x=%.10lf\\n\", z);
#{eq.collect {|n| "  printf(\"#{n}=%.10lf\\n\", y[#{n[7..-1].to_i - 1}]);"}.join("\n")}
  free(y);
  free_sys(sys);
  return 0;
}
EOF
    end
  end
end
