module Srnggen
  # generic module representing a primitive in the combstruct language
  module Comb
    # make all the necessary pre-calculations for the sampling
    def compile
      elems.each {|e| e.compile}
      compile_self
    end

    # store in @oracle the value for this node. This method is generally
    # overridden by subclasses.
    def compile_self
    end

    def indets h
      elems.each {|e| h = e.indets h}
      h
    end

    def to_sys
      elem.to_sys
    end

    module Grammar
      include Comb
    end

    # empty unary construction
    module Marker
      include Comb
      def compile_self
        @oracle = elem.oracle
      end
    end

    module Def
      include Comb
    end

    module Aliased
      def set_content x
        @alias = x
      end

      def indets h
        h[@alias] = nil
        h
      end

      def replace n, v
        if @alias == n
          extend Dereferenced
          set_content v
        end
      end
    end

    module Dereferenced
      def set_content x
        @elems << x
        @to_sys = nil
      end

      def to_sys
        @to_sys ||= elem.to_sys
      end

      def indets h
        elem.indets h
      end

      def replace n, v
        elem.replace n, v
      end
    end

    def replace n, v
      elems.each {|e| e.replace n, v}
    end

    # An element not contributing to the size of the object
    module Epsilon
      include Comb
    end

    # An element contributing <tt>weight</tt> to the size of the object
    module Atom
      include Comb
      NAME = "x"
      def weight
        1
      end

      def compile_self
        @oracle = doc.oracle[NAME]*weight
      end
    end

    class Mark < Node
      include Atom
      def initialize doc
        super nil, doc
      end
    end

    # A sequence of one or more constructions
    module Sequence
      include Comb
      def Sequence.geom i
        (Math.log(rand)/Math.log(i)).ceil
      end

      def compile_self
        @oracle = elem.oracle/(1 - elem.oracle)
        @magic = elem.oracle
      end
    end

    # The product of two constructions
    module Product
      include Comb
      def compile_self
        @oracle = 1
        elems.each {|e| @oracle *= e.oracle}
      end
    end
    
    # The union of two constructions
    module Union
      include Comb
      def compile_self
        @oracle, t, @magic = 0, 0, []
        elems.each {|e| @oracle += e.oracle}
        elems.each do |e|
          t += e.oracle
          @magic << t/oracle
        end
      end
    end

    # a reference to an element defined elsewhere
    module Reference
      include Comb
      def indets h
        h[ref] = nil
        h
      end

      def replace n, v
        if ref == n
          extend(v.kind_of?(Def) ? Dereferenced : Aliased)
          set_content v
        end
      end

      def compile_self
        @oracle = doc.oracle[ref]
      end
    end
  end
end
