module Srnggen
  # Class representing a hash table of the values of the oracle for each element
  class Oracle
    # source must be the name of a file containing name=value pairs for each
    # element or a Grammar for which the oracle will be calculated using Maple
    def initialize source
      source = calculate source if source.class == Document
      @values, @comments = {}, []
      IO.foreach source do |s|
        s.chomp!
        if s =~ /^#/
          @comments << s
        else
          k, v = s.split '='
          @values[k] = v.to_f
        end
      end
    end

    def to_s
      (@comments + @values.collect {|k, v| "#{k}=#{v}"}).join "\n"
    end

    # get the value of the oracle on a given element
    def [] x
      @values[x]
    end
  end
end
